// NODE.CPP: node definitions
//
// (c) Copyright, C Gazette 1990
// May be used freely as long as authorship and
//     publication are acknowledged
 
#include "node.hpp"
#include "sysio.hpp"
#include <stdio.h>
#include <conio.h>

node * node::next_valid() 
{
  if(cursor)   // return next node if there is one, else this node
  {
    if(cursor->next)   
       cursor = cursor->next;
    return cursor->data;
  }
  static node empty("programming error: empty node");
  return &empty;
}

// the following is slower than with a doubly-linked list, but it is
// only used for trimming a search, so the net result is probably faster

void node::remove(node * nd) 
{
  holder * cur, * drag;
  cur = drag = head;
  while(cur) 
  {
    if (cur->data == nd) 
    {
      if(cur == head) 
         head = head->next;  // move head forward
      drag->next = cur->next;  // Thread list past current element
      delete cur;
      return;
    }
    drag = cur;       // drag is always one behind
    cur = cur->next;  // go to next element (may be NULL)
  }
}

void node::conditional_pop()  // pop top if not the last one
{
  cursor = head;
  if(!head->next) 
     return;          // quit if no next element
  head = head->next;  // move the head down one
  delete cursor;      // delete the top element
  cursor = head;      // put the cursor at head again
}

void node::print()    // list contents of this node
{
  printf("----------\nnode identity: %s\nnode contents:\n", identity);
  node * cur = top();
  while(cur) 
  {
    puts(cur->identity);
    cur = next();
  }
}

void node::copy(node * rv) 
{
  node * n = rv->top();
  while(n) 
  { 
    add(n); 
    n = rv->next(); 
  }
}

node * node::contains(char * string)  // is string in the node list?
{
  node * cur = top();
  while(cur) 
  {
    if(!strcmp(cur->identity, string)) 
       return cur;
    cur = next();
  }
  return NULL;
}

int node::connected(node * n)  // is the pointer n in the node list?
{
  for(node * cur = top(); cur; cur = next())
    if(n == cur) 
       return 1;
  return 0;
}

void node::intersect(node * compare) 
{
  node * n = top();
  while(n) 
  {
    // find the nodes in common between this & compare:
    if(!compare->connected(n))
      remove(n);
    n = next();
  }
}

// turn node list into a numbered menu, get a number and return
// corresponding node pointer:
node * selection_menu(node * sn, char * msg) 
{
  const lines_per_page = 22;
  if(*msg) 
     puts(msg);
  int count = 0, linecount = 0;
  node * cur = sn->top();
  while(cur) 
  {
    printf("%3d) %-32s", count++, cur->identity);
    if(count % 2 == 0) 
    { 
      putchar('\n');
      if(++linecount % lines_per_page == 0) 
      {
        printf("any key to continue");
        get_key();
        putchar('\n');
      }
    }
    cur = sn->next();
  }
  if(count % 2) 
     putchar('\n');
  char buf[25];
  printf("->");
  int select = atoi(gets(buf));
  node * match = sn->top();
  while(select--) 
     match = sn->next();
  return match;
}